﻿-- Tabelle Filialen anlegen
CREATE TABLE Filialen (id serial, name varchar, kommentar varchar);
ALTER TABLE Filialen ADD CONSTRAINT pk_Filialen PRIMARY KEY(id);

-- Master-Tabelle anlegen
CREATE TABLE Inventar (id serial, filiale int, gegenstand varchar, anzahl float4);

-- Schema für die Partitionen anlegen
CREATE SCHEMA partitionen;

-- Trigger, der das Einfügen in die Master-Tabelle unterbindet
CREATE FUNCTION partitionsfehler() RETURNS "trigger" AS $$
    BEGIN
	RAISE EXCEPTION 'Partitionierung: Einfuegen nicht erlaubt';
    END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER tr_Inventar_insert
    BEFORE INSERT ON Inventar
    FOR EACH ROW
    EXECUTE PROCEDURE partitionsfehler();

-- Trigger zum Anlegen neuer Partitionen
CREATE FUNCTION addfiliale(v_id integer) RETURNS void AS $$
    DECLARE
	dst_schema varchar = 'partitionen';
    BEGIN
	execute 'CREATE TABLE ' || dst_schema || '.Inventar_' || v_id || '(CHECK (filiale='|| v_id || ')) INHERITS (Inventar);';
	execute 'ALTER TABLE ' || dst_schema || '.Inventar_' || v_id || ' ADD CONSTRAINT pk_Inventar_' || v_id || ' PRIMARY KEY(id);';
	execute 'ALTER TABLE ' || dst_schema || '.Inventar_' || v_id || ' ADD CONSTRAINT fk_Inventar_' || v_id || '_filiale FOREIGN KEY (filiale) REFERENCES Filialen (id) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;';
	execute 'CREATE RULE Inventar_' || v_id || '_insert AS ON INSERT TO Inventar WHERE (filiale=' || v_id || ') DO INSTEAD INSERT INTO ' || dst_schema || '.Inventar_' || v_id || ' VALUES (NEW.id, NEW.filiale, NEW.gegenstand, NEW.anzahl)';
    END;
$$ LANGUAGE plpgsql;

CREATE FUNCTION filiale_insert() RETURNS "trigger" AS $$
    BEGIN
	PERFORM addfiliale(NEW.id);
	RETURN NEW;
    END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER tr_Filiale_insert
    BEFORE INSERT ON Filialen
    FOR EACH ROW
    EXECUTE PROCEDURE filiale_insert();

