package de.steinwedel.echo2.example;

import nextapp.echo2.app.*;
import nextapp.echo2.app.componentxml.*;
import nextapp.echo2.app.event.*;
import nextapp.echo2.app.list.DefaultListModel;

public class SimpleCalc extends ApplicationInstance implements ActionListener {
	
	public static final StyleSheet DEFAULT_STYLE_SHEET; 
	
	private TextField value1;
	private TextField value2;
	private SelectField operator;
	private Button calculate;
	private TextField result;
	
	static { 
		try { 
			DEFAULT_STYLE_SHEET = StyleSheetLoader.load( "/resources/styles.xml", Thread.currentThread().getContextClassLoader());
		} catch (ComponentXmlException ex) { 
			throw new RuntimeException(ex); 
		} 
	}
	
	public Window init() {
		//Erzeuge Dialogfenster
		Window window = new Window();
		window.setTitle("Simple Calculator with Echo2");
		
		// Erzeuge Widgets
		value1 = new TextField();
		value2 = new TextField();
		operator = new SelectField();
		String[] operators = {"+", "-", "*","/"};
		operator.setModel(new DefaultListModel(operators));
		operator.setSelectedIndex(0);
		calculate = new Button("=");
		calculate.addActionListener(this);
		result = new TextField();
		
		// Platziere Widgets auf dem Dialogfenster
		Row layout = new Row();
		layout.add(value1);
		layout.add(operator);
		layout.add(value2);
		layout.add(calculate);
		layout.add(result);
		ContentPane mainContentPane = new ContentPane();
		mainContentPane.add(layout);
		window.setContent(mainContentPane);
		
		// Wende Styles an
		setStyleSheet(DEFAULT_STYLE_SHEET);
		setStyle(window);
		
		return window;
	}
	
	// Hier werden eingehende Events verarbeitet
	public void actionPerformed(ActionEvent event) {
		// Berechne Ergebnis
		try {
			float v1 = Float.parseFloat(value1.getText());
			float v2 = Float.parseFloat(value2.getText());
			float r;
			switch (operator.getSelectedItem().toString().charAt(0)) {
				case '-':
					r = v1 - v2;
					break;
				case '*':
					r = v1 * v2;
					break;
				case '/':
					r = v1 / v2;
					break;
				default:
					r = v1 + v2;
			}
			result.setText(Float.toString(r));
		} catch (Exception e) {
			result.setText("Fehlerhafte Eingabe!");
		}
	}
	
	// Gehe mit einer Rekursion ber alle Widgets
	// und wende den Default-Style an
	private void setStyle(Component component) {
		component.setStyleName("Default");
		for (int i = 0; i < component.getComponentCount(); i++)
			setStyle(component.getComponents()[i]);
	}

}
